@@ -97,7 +97,7 @@ InitVelocityData(DeviceVelocityPtr vel)
 {
     memset(vel, 0, sizeof(DeviceVelocityRec));
 
-    vel->corr_mul = 10.0;       /* dots per 10 milisecond should be usable */
+    vel->corr_mul = 10.0;       /* dots per 10 millisecond should be usable */
     vel->const_acceleration = 1.0;      /* no acceleration/deceleration  */
     vel->reset_time = 300;
     vel->use_softening = 1;
@@ -134,13 +134,20 @@ InitPredictableAccelerationScheme(DeviceIntPtr dev,
     scheme = *protoScheme;
     vel = calloc(1, sizeof(DeviceVelocityRec));
     schemeData = calloc(1, sizeof(PredictableAccelSchemeRec));
-    if (!vel || !schemeData)
+    if (!vel || !schemeData) {
+        free(vel);
+        free(schemeData);
         return FALSE;
+    }
     InitVelocityData(vel);
     schemeData->vel = vel;
     scheme.accelData = schemeData;
-    if (!InitializePredictableAccelerationProperties(dev, vel, schemeData))
+    if (!InitializePredictableAccelerationProperties(dev, vel, schemeData)) {
+        FreeVelocityData(vel);
+        free(vel);
+        free(schemeData);
         return FALSE;
+    }
     /* all fine, assign scheme to device */
     dev->valuator->accelScheme = scheme;
     return TRUE;
@@ -159,7 +166,7 @@ AccelerationDefaultCleanup(DeviceIntPtr dev)
          * AccelSchemeProc(), but that seems impossible. Schemes don't get
          * switched often anyway.
          */
-        OsBlockSignals();
+        input_lock();
         dev->valuator->accelScheme.AccelSchemeProc = NULL;
         FreeVelocityData(vel);
         free(vel);
@@ -169,7 +176,7 @@ AccelerationDefaultCleanup(DeviceIntPtr dev)
                                                 accelData);
         free(dev->valuator->accelScheme.accelData);
         dev->valuator->accelScheme.accelData = NULL;
-        OsReleaseSignals();
+        input_unlock();
     }
 }
 
@@ -245,11 +252,10 @@ AccelSetDecelProperty(DeviceIntPtr dev, Atom atom,
     if (checkOnly) {
         if (rc)
             return rc;
-        return (v >= 1.0f) ? Success : BadValue;
+        return (v > 0) ? Success : BadValue;
     }
 
-    if (v >= 1.0f)
-        vel->const_acceleration = 1 / v;
+    vel->const_acceleration = 1 / v;
 
     return Success;
 }
@@ -514,7 +520,7 @@ DoGetDirection(int dx, int dy)
 
 /* cache DoGetDirection().
  * To avoid excessive use of direction calculation, cache the values for
- * [-5..5] for both x/y. Anything outside of that is calcualted on the fly.
+ * [-5..5] for both x/y. Anything outside of that is calculated on the fly.
  *
  * @return A bitmask for N, NE, S, SE, etc. indicating the directions for
  * this movement.
@@ -592,7 +598,7 @@ CalcTracker(const MotionTracker * tracker, int cur_t)
 /* find the most plausible velocity. That is, the most distant
  * (in time) tracker which isn't too old, the movement vector was
  * in the same octant, and where the velocity is within an
- * acceptable range to the inital velocity.
+ * acceptable range to the initial velocity.
  *
  * @return The tracker's velocity or 0 if the above conditions are unmet
  */
@@ -789,12 +795,12 @@ ComputeAcceleration(DeviceIntPtr dev,
             BasicComputeAcceleration(dev, vel, vel->last_velocity, threshold,
                                      acc);
         result +=
-            4.0f * BasicComputeAcceleration(dev, vel,
+            4.0 * BasicComputeAcceleration(dev, vel,
                                             (vel->last_velocity +
                                              vel->velocity) / 2,
                                             threshold,
                                             acc);
-        result /= 6.0f;
+        result /= 6.0;
         DebugAccelF("profile average [%.2f ... %.2f] is %.3f\n",
                     vel->velocity, vel->last_velocity, result);
     }
@@ -855,7 +861,7 @@ PowerProfile(DeviceIntPtr dev,
 {
     double vel_dist;
 
-    acc = (acc - 1.0) * 0.1f + 1.0;     /* without this, acc of 2 is unuseable */
+    acc = (acc - 1.0) * 0.1 + 1.0;     /* without this, acc of 2 is unuseable */
 
     if (velocity <= threshold)
         return vel->min_acceleration;
@@ -873,9 +879,9 @@ PowerProfile(DeviceIntPtr dev,
 static inline double
 CalcPenumbralGradient(double x)
 {
-    x *= 2.0f;
-    x -= 1.0f;
-    return 0.5f + (x * sqrt(1.0 - x * x) + asin(x)) / M_PI;
+    x *= 2.0;
+    x -= 1.0;
+    return 0.5 + (x * sqrt(1.0 - x * x) + asin(x)) / M_PI;
 }
 
 /**
@@ -911,23 +917,23 @@ SmoothLinearProfile(DeviceIntPtr dev,
 {
     double res, nv;
 
-    if (acc > 1.0f)
-        acc -= 1.0f;            /*this is so acc = 1 is no acceleration */
+    if (acc > 1.0)
+        acc -= 1.0;            /*this is so acc = 1 is no acceleration */
     else
-        return 1.0f;
+        return 1.0;
 
-    nv = (velocity - threshold) * acc * 0.5f;
+    nv = (velocity - threshold) * acc * 0.5;
 
     if (nv < 0) {
         res = 0;
     }
     else if (nv < 2) {
-        res = CalcPenumbralGradient(nv * 0.25f) * 2.0f;
+        res = CalcPenumbralGradient(nv * 0.25) * 2.0;
     }
     else {
-        nv -= 2.0f;
-        res = nv * 2.0f / M_PI  /* steepness of gradient at 0.5 */
-            + 1.0f;             /* gradient crosses 2|1 */
+        nv -= 2.0;
+        res = nv * 2.0 / M_PI  /* steepness of gradient at 0.5 */
+            + 1.0;             /* gradient crosses 2|1 */
     }
     res += vel->min_acceleration;
     return res;
@@ -944,7 +950,7 @@ SmoothLimitedProfile(DeviceIntPtr dev,
 {
     double res;
 
-    if (velocity >= threshold || threshold == 0.0f)
+    if (velocity >= threshold || threshold == 0.0)
         return acc;
 
     velocity /= threshold;      /* should be [0..1[ now */
@@ -966,7 +972,7 @@ static double
 NoProfile(DeviceIntPtr dev,
           DeviceVelocityPtr vel, double velocity, double threshold, double acc)
 {
-    return 1.0f;
+    return 1.0;
 }
 
 static PointerAccelerationProfileFunc
@@ -1086,7 +1092,7 @@ acceleratePointerPredictable(DeviceIntPtr dev, ValuatorMask *val, CARD32 evtime)
         return;
 
     if (velocitydata->statistics.profile_number == AccelProfileNone &&
-        velocitydata->const_acceleration == 1.0f) {
+        velocitydata->const_acceleration == 1.0) {
         return;                 /*we're inactive anyway, so skip the whole thing. */
     }
 
@@ -1114,8 +1120,8 @@ acceleratePointerPredictable(DeviceIntPtr dev, ValuatorMask *val, CARD32 evtime)
                                        (double) dev->ptrfeed->ctrl.den);
 
             DebugAccelF("mult is %f\n", mult);
-            if (mult != 1.0f || velocitydata->const_acceleration != 1.0f) {
-                if (mult > 1.0f && soften)
+            if (mult != 1.0 || velocitydata->const_acceleration != 1.0) {
+                if (mult > 1.0 && soften)
                     ApplySoftening(velocitydata, &dx, &dy);
                 ApplyConstantDeceleration(velocitydata, &dx, &dy);
 
