@@ -38,14 +38,12 @@
 #ifdef WITH_LIBDRM
 #include <xf86drm.h>
 #endif
-#include "xf86Module.h"
 #include "list.h"
 #include "scrnintstr.h"
 #include "windowstr.h"
 #include "dixstruct.h"
 #include "dri2.h"
 #include "dri2int.h"
-#include "xf86VGAarbiter.h"
 #include "damage.h"
 #include "xf86.h"
 
@@ -100,7 +98,7 @@ typedef struct _DRI2Drawable {
     unsigned blocked[3];
     Bool needInvalidate;
     int prime_id;
-    PixmapPtr prime_slave_pixmap;
+    PixmapPtr prime_secondary_pixmap;
     PixmapPtr redirectpixmap;
 } DRI2DrawableRec, *DRI2DrawablePtr;
 
@@ -183,30 +181,33 @@ DRI2GetScreen(ScreenPtr pScreen)
 }
 
 static ScreenPtr
-GetScreenPrime(ScreenPtr master, int prime_id)
+GetScreenPrime(ScreenPtr primary, int prime_id)
 {
-    ScreenPtr slave;
-    if (prime_id == 0 || xorg_list_is_empty(&master->offload_slave_list)) {
-        return master;
+    ScreenPtr secondary;
+    if (prime_id == 0) {
+        return primary;
     }
-    xorg_list_for_each_entry(slave, &master->offload_slave_list, offload_head) {
+    xorg_list_for_each_entry(secondary, &primary->secondary_list, secondary_head) {
         DRI2ScreenPtr ds;
 
-        ds = DRI2GetScreen(slave);
+        if (!secondary->is_offload_secondary)
+            continue;
+
+        ds = DRI2GetScreen(secondary);
         if (ds == NULL)
             continue;
 
         if (ds->prime_id == prime_id)
-            return slave;
+            return secondary;
     }
-    return master;
+    return primary;
 }
 
 static DRI2ScreenPtr
-DRI2GetScreenPrime(ScreenPtr master, int prime_id)
+DRI2GetScreenPrime(ScreenPtr primary, int prime_id)
 {
-    ScreenPtr slave = GetScreenPrime(master, prime_id);
-    return DRI2GetScreen(slave);
+    ScreenPtr secondary = GetScreenPrime(primary, prime_id);
+    return DRI2GetScreen(secondary);
 }
 
 static DRI2DrawablePtr
@@ -261,7 +262,7 @@ DRI2AllocateDrawable(DrawablePtr pDraw)
     xorg_list_init(&pPriv->reference_list);
     pPriv->needInvalidate = FALSE;
     pPriv->redirectpixmap = NULL;
-    pPriv->prime_slave_pixmap = NULL;
+    pPriv->prime_secondary_pixmap = NULL;
     if (pDraw->type == DRAWABLE_WINDOW) {
         pWin = (WindowPtr) pDraw;
         dixSetPrivate(&pWin->devPrivates, dri2WindowPrivateKey, pPriv);
@@ -426,9 +427,9 @@ DRI2DrawableGone(void *p, XID id)
         dixSetPrivate(&pPixmap->devPrivates, dri2PixmapPrivateKey, NULL);
     }
 
-    if (pPriv->prime_slave_pixmap) {
-        (*pPriv->prime_slave_pixmap->master_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_slave_pixmap->master_pixmap);
-        (*pPriv->prime_slave_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_slave_pixmap);
+    if (pPriv->prime_secondary_pixmap) {
+        (*pPriv->prime_secondary_pixmap->primary_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_secondary_pixmap->primary_pixmap);
+        (*pPriv->prime_secondary_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_secondary_pixmap);
     }
 
     if (pPriv->buffers != NULL) {
@@ -814,10 +815,10 @@ DrawablePtr DRI2UpdatePrime(DrawablePtr pDraw, DRI2BufferPtr pDest)
     DRI2DrawablePtr pPriv = DRI2GetDrawable(pDraw);
     PixmapPtr spix;
     PixmapPtr mpix = GetDrawablePixmap(pDraw);
-    ScreenPtr master, slave;
+    ScreenPtr primary, secondary;
     Bool ret;
 
-    master = mpix->drawable.pScreen;
+    primary = mpix->drawable.pScreen;
 
     if (pDraw->type == DRAWABLE_WINDOW) {
         WindowPtr pWin = (WindowPtr)pDraw;
@@ -830,15 +831,15 @@ DrawablePtr DRI2UpdatePrime(DrawablePtr pDraw, DRI2BufferPtr pDest)
                 pPriv->redirectpixmap->drawable.depth == pDraw->depth) {
                 mpix = pPriv->redirectpixmap;
             } else {
-                if (master->ReplaceScanoutPixmap) {
-                    mpix = (*master->CreatePixmap)(master, pDraw->width, pDraw->height,
+                if (primary->ReplaceScanoutPixmap) {
+                    mpix = (*primary->CreatePixmap)(primary, pDraw->width, pDraw->height,
                                                    pDraw->depth, CREATE_PIXMAP_USAGE_SHARED);
                     if (!mpix)
                         return NULL;
 
-                    ret = (*master->ReplaceScanoutPixmap)(pDraw, mpix, TRUE);
+                    ret = (*primary->ReplaceScanoutPixmap)(pDraw, mpix, TRUE);
                     if (ret == FALSE) {
-                        (*master->DestroyPixmap)(mpix);
+                        (*primary->DestroyPixmap)(mpix);
                         return NULL;
                     }
                     pPriv->redirectpixmap = mpix;
@@ -846,30 +847,31 @@ DrawablePtr DRI2UpdatePrime(DrawablePtr pDraw, DRI2BufferPtr pDest)
                     return NULL;
             }
         } else if (pPriv->redirectpixmap) {
-            (*master->ReplaceScanoutPixmap)(pDraw, pPriv->redirectpixmap, FALSE);
-            (*master->DestroyPixmap)(pPriv->redirectpixmap);
+            (*primary->ReplaceScanoutPixmap)(pDraw, pPriv->redirectpixmap, FALSE);
+            (*primary->DestroyPixmap)(pPriv->redirectpixmap);
             pPriv->redirectpixmap = NULL;
         }
     }
 
-    slave = GetScreenPrime(pDraw->pScreen, pPriv->prime_id);
+    secondary = GetScreenPrime(pDraw->pScreen, pPriv->prime_id);
 
     /* check if the pixmap is still fine */
-    if (pPriv->prime_slave_pixmap) {
-        if (pPriv->prime_slave_pixmap->master_pixmap == mpix)
-            return &pPriv->prime_slave_pixmap->drawable;
+    if (pPriv->prime_secondary_pixmap) {
+        if (pPriv->prime_secondary_pixmap->primary_pixmap == mpix)
+            return &pPriv->prime_secondary_pixmap->drawable;
         else {
-            (*pPriv->prime_slave_pixmap->master_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_slave_pixmap->master_pixmap);
-            (*slave->DestroyPixmap)(pPriv->prime_slave_pixmap);
-            pPriv->prime_slave_pixmap = NULL;
+            PixmapUnshareSecondaryPixmap(pPriv->prime_secondary_pixmap);
+            (*pPriv->prime_secondary_pixmap->primary_pixmap->drawable.pScreen->DestroyPixmap)(pPriv->prime_secondary_pixmap->primary_pixmap);
+            (*secondary->DestroyPixmap)(pPriv->prime_secondary_pixmap);
+            pPriv->prime_secondary_pixmap = NULL;
         }
     }
 
-    spix = PixmapShareToSlave(mpix, slave);
+    spix = PixmapShareToSecondary(mpix, secondary);
     if (!spix)
         return NULL;
 
-    pPriv->prime_slave_pixmap = spix;
+    pPriv->prime_secondary_pixmap = spix;
 #ifdef COMPOSITE
     spix->screen_x = mpix->screen_x;
     spix->screen_y = mpix->screen_y;
@@ -1436,21 +1438,17 @@ get_prime_id(void)
 static char *
 dri2_probe_driver_name(ScreenPtr pScreen, DRI2InfoPtr info)
 {
-    ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
-    EntityInfoPtr pEnt = xf86GetEntityInfo(pScrn->entityList[0]);
-    struct pci_device *pdev = NULL;
+#ifdef WITH_LIBDRM
     int i, j;
+    char *driver = NULL;
+    drmDevicePtr dev;
 
-    if (pEnt)
-        pdev = xf86GetPciInfoForEntity(pEnt->index);
-
-    /* For non-PCI devices, just assume that the 3D driver is named
-     * the same as the kernel driver.  This is currently true for vc4
-     * and msm (freedreno).
+    /* For non-PCI devices and drmGetDevice fail, just assume that
+     * the 3D driver is named the same as the kernel driver. This is
+     * currently true for vc4 and msm (freedreno).
      */
-    if (!pdev) {
+    if (drmGetDevice(info->fd, &dev) || dev->bustype != DRM_BUS_PCI) {
         drmVersionPtr version = drmGetVersion(info->fd);
-        char *kernel_driver;
 
         if (!version) {
             xf86DrvMsg(pScreen->myNum, X_ERROR,
@@ -1459,29 +1457,38 @@ dri2_probe_driver_name(ScreenPtr pScreen, DRI2InfoPtr info)
             return NULL;
         }
 
-        kernel_driver = strndup(version->name, version->name_len);
+        driver = strndup(version->name, version->name_len);
         drmFreeVersion(version);
-        return kernel_driver;
+        return driver;
     }
 
     for (i = 0; driver_map[i].driver; i++) {
-        if (pdev->vendor_id != driver_map[i].vendor_id)
+        if (dev->deviceinfo.pci->vendor_id != driver_map[i].vendor_id)
             continue;
 
-        if (driver_map[i].num_chips_ids == -1)
-            return strdup(driver_map[i].driver);
+        if (driver_map[i].num_chips_ids == -1) {
+             driver = strdup(driver_map[i].driver);
+             goto out;
+        }
 
         for (j = 0; j < driver_map[i].num_chips_ids; j++) {
-            if (driver_map[i].chip_ids[j] == pdev->device_id)
-                return strdup(driver_map[i].driver);
+            if (driver_map[i].chip_ids[j] == dev->deviceinfo.pci->device_id) {
+                driver = strdup(driver_map[i].driver);
+                goto out;
+            }
         }
     }
 
     xf86DrvMsg(pScreen->myNum, X_ERROR,
                "[DRI2] No driver mapping found for PCI device "
                "0x%04x / 0x%04x\n",
-               pdev->vendor_id, pdev->device_id);
+               dev->deviceinfo.pci->vendor_id, dev->deviceinfo.pci->device_id);
+out:
+    drmFreeDevice(&dev);
+    return driver;
+#else
     return NULL;
+#endif
 }
 
 Bool
@@ -1499,12 +1506,6 @@ DRI2ScreenInit(ScreenPtr pScreen, DRI2InfoPtr info)
     if (info->version < 3)
         return FALSE;
 
-    if (!xf86VGAarbiterAllowDRI(pScreen)) {
-        xf86DrvMsg(pScreen->myNum, X_WARNING,
-                   "[DRI2] Direct rendering is not supported when VGA arb is necessary for the device\n");
-        return FALSE;
-    }
-
     if (!dixRegisterPrivateKey(&dri2ScreenPrivateKeyRec, PRIVATE_SCREEN, 0))
         return FALSE;
 
@@ -1529,6 +1530,7 @@ DRI2ScreenInit(ScreenPtr pScreen, DRI2InfoPtr info)
     ds->CreateBuffer = info->CreateBuffer;
     ds->DestroyBuffer = info->DestroyBuffer;
     ds->CopyRegion = info->CopyRegion;
+    cur_minor = 1;
 
     if (info->version >= 4) {
         ds->ScheduleSwap = info->ScheduleSwap;
@@ -1536,13 +1538,7 @@ DRI2ScreenInit(ScreenPtr pScreen, DRI2InfoPtr info)
         ds->GetMSC = info->GetMSC;
         cur_minor = 3;
     }
-    else {
-        cur_minor = 1;
-    }
 
-    if (info->version >= 8) {
-        ds->AuthMagic = info->AuthMagic2;
-    }
     if (info->version >= 5) {
         ds->LegacyAuthMagic = info->AuthMagic;
     }
@@ -1557,6 +1553,10 @@ DRI2ScreenInit(ScreenPtr pScreen, DRI2InfoPtr info)
         cur_minor = 4;
     }
 
+    if (info->version >= 8) {
+        ds->AuthMagic = info->AuthMagic2;
+    }
+
     if (info->version >= 9) {
         ds->CreateBuffer2 = info->CreateBuffer2;
         if (info->CreateBuffer2 && pScreen->isGPU) {
@@ -1602,9 +1602,16 @@ DRI2ScreenInit(ScreenPtr pScreen, DRI2InfoPtr info)
         if (info->driverName) {
             ds->driverNames[0] = info->driverName;
         } else {
+            /* FIXME dri2_probe_driver_name() returns a strdup-ed string,
+             * currently this gets leaked */
             ds->driverNames[0] = ds->driverNames[1] = dri2_probe_driver_name(pScreen, info);
             if (!ds->driverNames[0])
                 return FALSE;
+
+            /* There is no VDPAU driver for i965, fallback to the generic
+             * OpenGL/VAAPI va_gl backend to emulate VDPAU on i965. */
+            if (strcmp(ds->driverNames[0], "i965") == 0)
+                ds->driverNames[1] = "va_gl";
         }
     }
     else {
@@ -1625,7 +1632,7 @@ DRI2ScreenInit(ScreenPtr pScreen, DRI2InfoPtr info)
     pScreen->SetWindowPixmap = DRI2SetWindowPixmap;
 
     xf86DrvMsg(pScreen->myNum, X_INFO, "[DRI2] Setup complete\n");
-    for (i = 0; i < sizeof(driverTypeNames) / sizeof(driverTypeNames[0]); i++) {
+    for (i = 0; i < ARRAY_SIZE(driverTypeNames); i++) {
         if (i < ds->numDrivers && ds->driverNames[i]) {
             xf86DrvMsg(pScreen->myNum, X_INFO, "[DRI2]   %s driver: %s\n",
                        driverTypeNames[i], ds->driverNames[i]);
